/* Declarations for natsu.cpp.
   Copyright (C) 2002,03 Yamamoto Takanori / Tatyana.

This is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this software; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <direct.h>

size_t strlcpy(char *dst, const char *src, size_t siz);
size_t strlcat(char *dst, const char *src, size_t siz);
void PrepareDirTree(const char *s0);
void func(const char *src, const char *dst, size_t start, size_t size);

size_t strlcpy(char *dst, const char *src, size_t siz)
{
	register char *d = dst;
	register const char *s = src;
	register size_t n = siz;

	/* Copy as many bytes as will fit */
	if (n != 0 && --n != 0) {
		do {
			if ((*d++ = *s++) == 0)
				break;
		} while (--n != 0);
	}

	/* Not enough room in dst, add NUL and traverse rest of src */
	if (n == 0) {
		if (siz != 0)
			*d = '\0';		/* NUL-terminate dst */
		while (*s++)
			;
	}

	return(s - src - 1);	/* count does not include NUL */
}

size_t strlcat(char *dst, const char *src, size_t siz)
{
	register char *d = dst;
	register const char *s = src;
	register size_t n = siz;
	size_t dlen;

	/* Find the end of dst and adjust bytes left but don't go past end */
	while (n-- != 0 && *d != '\0')
		d++;
	dlen = d - dst;
	n = siz - dlen;

	if (n == 0)
		return(dlen + strlen(s));
	while (*s != '\0') {
		if (n != 1) {
			*d++ = *s;
			n--;
		}
		s++;
	}
	*d = '\0';

	return(dlen + (s - src));	/* count does not include NUL */
}

void PrepareDirTree(const char *s0)
{
	size_t len = strlen(s0)+1;
	char *s, *t;
	if((s=(char *)malloc(len+1))==NULL || (t=(char *)malloc(len+1))==NULL){
		fprintf(stderr, "not enough memory.\n");
		exit(0);
	}
	strlcpy(s, s0, len+1);
	strlcpy(t, "", len+1);

	char *p = s;
	while(strstr(p, "\\")){
		char *q = strtok(p, "\\");
		p=p+strlen(p)+1;
		strlcat(t, q, len+1);
		strlcat(t, "\\", len+1);
		if(mkdir(t)){
			/* mkdir failed */
		}
	}

	free(t);
	free(s);
}

void func(const char *src, const char *dst, size_t start, size_t size){

	PrepareDirTree(dst);

	FILE *fp_src, *fp_dst;
	if(((fp_src=fopen(src, "rb"))==NULL) || ((fp_dst=fopen(dst, "wb"))==NULL)){
		fprintf(stderr, "file open error.\n");
		exit(0);
	}

	fseek(fp_src, start, SEEK_SET);

	unsigned char *buff = (unsigned char *)malloc(size);
	if(buff==NULL){
		fprintf(stderr, "not enough memory.\n");
		exit(0);
	}
	if(fread(buff, 1, size, fp_src) != size){
		fprintf(stderr, "file read error.\n");
		exit(0);
	}
	if(fwrite(buff, 1, size, fp_dst) != size){
		fprintf(stderr, "file write error.\n");
		exit(0);
	}

	free(buff);
	fclose(fp_dst);
	fclose(fp_src);
}

int main(int argc, char *argv[]){

	if(argc==1){
		fprintf(stderr, "usage : %s (archive)\n", argv[0]);
		return 0;
	}

	FILE *fp = fopen(argv[1], "rb");
	if(fp == NULL){
		fprintf(stderr, "file open error.\n");
		return -1;
	}

	unsigned int header_lastaddr;
	size_t size;

	if((size=fread(&header_lastaddr, sizeof(header_lastaddr), 1, fp)) != 1){
		fprintf(stderr, "read error.\n");
		return -1;
	}
	printf("H.LASTADDR : %04x\n--------\n", header_lastaddr);

	struct idx{
		unsigned int p, q, r, s;
	};

	while(ftell(fp)<header_lastaddr){
		struct idx i;
		if((size=fread(&i, sizeof(i), 1, fp)) != 1){
			fprintf(stderr, "read error.\n");
			return -1;
		}
		printf("PARAM : 0x%04x,0x%04x,0x%04x,0x%04x\n", i.p,i.q,i.r,i.s);

		char fn[256]; int n=0;
		for(;;){
			char c;
			if((size=fread(&c, 1, 1, fp)) != 1){
				fprintf(stderr, "read error.\n");
				return -1;
			}
			fn[n++]=c;
			if(c==NULL){break;}
			if(n==256){
				fprintf(stderr, "file name is too long.\n");
				return -1;
			}
		}
		printf("FNAME : %s\n--------\n", fn);

		func(argv[1], fn, i.r, i.s);
	}


	fclose(fp);
	return 0;
}


